import Vue from 'vue';

const getCurrentPosition = options => new Promise((resolve, reject) => navigator.geolocation.getCurrentPosition(resolve, reject, options));
const positionToObject = ({
  coords,
  timestamp
}) => ({
  coords: {
    latitude: coords.latitude,
    longitude: coords.longitude,
    altitude: coords.altitude,
    accuracy: coords.accuracy,
    altitudeAccuracy: coords.altitudeAccuracy,
    heading: coords.heading,
    speed: coords.speed
  },
  timestamp
});

const VueGeolocation = Vue.extend({
  data: () => ({
    position: null,
    options: {},
    _interval: null,
    interval: 60,
    loading: false
  }),
  computed: {
    supported() {
      return 'geolocation' in navigator;
    },

    coords() {
      const {
        position
      } = this;
      return position ? position.coords : null;
    },

    timestamp() {
      const {
        position
      } = this;
      return position ? position.coords : null;
    },

    geoJSON() {
      const {
        coords
      } = this;
      if (!coords) return null;
      const {
        latitude,
        longitude,
        altitude
      } = coords;
      const coordinates = [latitude, longitude];
      return altitude === null ? coordinates : [...coordinates, altitude];
    }

  },
  watch: {
    interval: 'watch'
  },

  beforeDestroy() {
    this.clearTimer();
  },

  created() {
    console.log('geo');
    if (!process.client) return;
    console.log('geo2');
    this.watch(true);
  },

  methods: {
    watch(mandatory) {
      const {
        interval
      } = this;
      this.clearTimer();
      if (!interval) return;
      this.$nextTick(() => {
        this._interval = setInterval(() => this.getCurrentPosition(), interval * 1000);
        if (mandatory) this.getCurrentPosition();
      });
    },

    clearTimer() {
      const {
        _interval
      } = this;
      if (!_interval) return;
      clearInterval(_interval);
    },

    async getCurrentPosition(options) {
      if (!this.suported) return;
      const params = Object.assign({}, this.options, options || {});
      this.loading = true;
      const response = await getCurrentPosition(params);
      const position = positionToObject(response);
      this.position = position;
      this.loading = false;
      return position;
    }

  }
});

function install(Vue$$1) {
  if (install.installed) return;
  install.installed = true;
  Vue$$1.$geolocation = new VueGeolocation();
}
const plugin = {
  install
};
let GlobalVue = null;

if (typeof window !== 'undefined') {
  GlobalVue = window.Vue;
} else if (typeof global !== 'undefined') {
  GlobalVue = global.Vue;
}

if (GlobalVue) {
  GlobalVue.use(plugin);
}

export default VueGeolocation;
export { install };
